use actix_web::{get, web, Responder, http::header::ContentType, HttpResponse, post};
use chrono::{DateTime, ParseError};
use uuid::Uuid;

use gamenight_database::{gamenight::Gamenight, DbPool, GetConnection};

use crate::request::authorization::AuthUser;
use crate::request::requests::GamenightGet;
use crate::request::requests::GamenightPost;
use crate::request::responses::GameNightsResponse;
use crate::request::error::ApiError;


impl GamenightPost {
    pub fn into_with_user(&self, user: AuthUser) -> Result<Gamenight, ParseError> {
        Ok(Gamenight {
            datetime: DateTime::parse_from_rfc3339(&self.datetime)?.with_timezone(&chrono::Utc),
            id: Uuid::new_v4(),
            name: self.name.clone(),
            owner_id: user.id
        })
    }
}

impl From<GamenightGet> for Uuid {
    fn from(value: GamenightGet) -> Self {
        Uuid::parse_str(value.id.as_str()).unwrap()
    }
}

#[get("/gamenights")]
pub async fn gamenights(pool: web::Data<DbPool>, _user: AuthUser) -> Result<impl Responder, ApiError> {
    let mut conn = pool.get_conn();
    let gamenights: GameNightsResponse = gamenight_database::gamenights(&mut conn)?;

    Ok(HttpResponse::Ok()
        .content_type(ContentType::json())
        .body(serde_json::to_string(&gamenights)?)
    )
}

#[post("/gamenight")]
pub async fn gamenight_post(pool: web::Data<DbPool>, user: AuthUser, gamenight_data: web::Json<GamenightPost>) -> Result<impl Responder, ApiError> {
    let mut conn = pool.get_conn();
    
    gamenight_database::gamenight::add_gamenight(&mut conn, gamenight_data.into_with_user(user)?)?;
    
    Ok(HttpResponse::Ok())
}

#[get("/gamenight")]
pub async fn gamenight_get(pool: web::Data<DbPool>, _user: AuthUser, gamenight_data: web::Json<GamenightGet>) -> Result<impl Responder, ApiError> {
    let mut conn = pool.get_conn();

    let gamenight = gamenight_database::gamenight::get_gamenight(&mut conn, gamenight_data.into_inner().into())?;
    //let participants = schema::user::get_participants(&mut conn, gamenight_id);

    Ok(HttpResponse::Ok()
        .content_type(ContentType::json())
        .body(serde_json::to_string(&gamenight)?))
}