use gamenight_database::{DbPool, GetConnection, user::count_users_with_username, user::count_users_with_email};
use serde::{Serialize, Deserialize};
use validator::{Validate, ValidationError};

#[derive(Serialize, Deserialize, Clone)]
pub struct Login {
    pub username: String,
    pub password: String
}

pub struct RegisterContext<'v_a> {
    pub pool: &'v_a DbPool
}

pub fn unique_username(username: &String, context: &RegisterContext) -> Result<(), ValidationError> {
    let mut conn = context.pool.get_conn();

    match count_users_with_username(&mut conn, username)
    {
        Ok(0) => Ok(()),
        Ok(_) => Err(ValidationError::new("User already exists")),
        Err(_) => Err(ValidationError::new("Database error while validating user")),
    }
}

pub fn unique_email(email: &String, context: &RegisterContext) -> Result<(), ValidationError> {
    let mut conn = context.pool.get_conn();

    match count_users_with_email(&mut conn, email)
    {
        Ok(0) => Ok(()),
        Ok(_) => Err(ValidationError::new("email already exists")),
        Err(_) => Err(ValidationError::new("Database error while validating email"))
    }
}

#[derive(Serialize, Deserialize, Clone, Validate)]
#[validate(context = RegisterContext::<'v_a>)]
pub struct Register {
    #[validate(
        length(min = 1),
        custom(function = "unique_username", use_context)
    )]
    pub username: String,
    #[validate(
        email,
        custom(function = "unique_email", use_context)
    )]
    pub email: String,
    #[validate(length(min = 10), must_match(other = "password_repeat", ))]
    pub password: String,
    pub password_repeat: String,
}

#[derive(Serialize, Deserialize, Clone)]
pub struct GamenightPost {
    pub name: String,
    pub datetime: String
}

#[derive(Serialize, Deserialize, Clone)]
pub struct GamenightGet {
    pub id: String
}